@@ -22,25 +22,33 @@
 #include <unistd.h>
 #include <QString>
 
+class QColor;
 class QFile;
-class QString;
+class QVariant;
 
 namespace ConfigEditor
 {
-    void setGtk2ConfigValue(const QString &paramName, const QString &paramValue);
-    void setGtk3ConfigValueDconf(const QString &paramName, bool paramValue, const QString &category = QStringLiteral("org.gnome.desktop.interface"));
-    void setGtk3ConfigValueDconf(const QString &paramName, const QString &paramValue, const QString &category = QStringLiteral("org.gnome.desktop.interface"));
-    void setGtk3ConfigValueSettingsIni(const QString &paramName, const QString &paramValue);
-    void setGtk3ConfigValueXSettingsd(const QString &paramName, const QString &paramValue);
+    void setGtk2ConfigValue(const QString &paramName, const QVariant &paramValue);
+
+    void setGtk3ConfigValueGSettings(const QString &paramName, const QVariant &paramValue, const QString &category = QStringLiteral("org.gnome.desktop.interface"));
+    void setGtk3ConfigValueGSettingsAsEnum(const QString &paramName, int paramValue, const QString &category = QStringLiteral("org.gnome.desktop.interface"));
+    void setGtk3ConfigValueSettingsIni(const QString &paramName, const QVariant &paramValue);
+    void setGtk3ConfigValueXSettingsd(const QString &paramName, const QVariant &paramValue);
+
+    void setGtk3Colors(const QMap<QString, QColor> &colorsDefinitions);
 
     QString gtk2ConfigValue(const QString& paramName);
     QString gtk3ConfigValueSettingsIni(const QString& paramName);
 
     void removeLegacyGtk2Strings();
 
+    void addGtkModule(const QString &moduleName);
 
-    void replaceValueInGtkrcContents(QString &gtkrcContents, const QString &paramName, const QString &paramValue);
-    void replaceValueInXSettingsdContents(QString &xSettingsdContents, const QString &paramName, const QString &paramValue);
+    void addImportStatementToGtkCssUserFile();
+    void modifyColorsCssFile(const QMap<QString, QColor> &colorsDefinitions);
+
+    void replaceValueInGtkrcContents(QString &gtkrcContents, const QString &paramName, const QVariant &paramValue);
+    void replaceValueInXSettingsdContents(QString &xSettingsdContents, const QString &paramName, const QVariant &paramValue);
 
     QString readFileContents(QFile &gtkrc);
 
