@@ -27,6 +27,7 @@
 #include <KIconLoader>
 #include <KPluginFactory>
 #include <KConfigWatcher>
+#include <KColorScheme>
 
 #include "gtkconfig.h"
 #include "configvalueprovider.h"
@@ -39,24 +40,16 @@ GtkConfig::GtkConfig(QObject *parent, const QVariantList&) :
     configValueProvider(new ConfigValueProvider()),
     themePreviewer(new ThemePreviewer(this)),
     kdeglobalsConfigWatcher(KConfigWatcher::create(KSharedConfig::openConfig(QStringLiteral("kdeglobals")))),
-    kwinConfigWatcher(KConfigWatcher::create(KSharedConfig::openConfig(QStringLiteral("kwinrc"))))
+    kwinConfigWatcher(KConfigWatcher::create(KSharedConfig::openConfig(QStringLiteral("kwinrc")))),
+    kcminputConfigWatcher(KConfigWatcher::create(KSharedConfig::openConfig(QStringLiteral("kcminputrc"))))
 {
     QDBusConnection dbus = QDBusConnection::sessionBus();
     dbus.registerService(QStringLiteral("org.kde.GtkConfig"));
     dbus.registerObject(QStringLiteral("/GtkConfig"), this, QDBusConnection::ExportScriptableSlots);
 
-    connect(qGuiApp, &QGuiApplication::fontChanged, this, &GtkConfig::setFont);
-    connect(KIconLoader::global(), &KIconLoader::iconChanged, this, &GtkConfig::setIconTheme);
     connect(kdeglobalsConfigWatcher.data(), &KConfigWatcher::configChanged, this, &GtkConfig::onKdeglobalsSettingsChange);
     connect(kwinConfigWatcher.data(), &KConfigWatcher::configChanged, this, &GtkConfig::onKWinSettingsChange);
-    dbus.connect(
-        QString(),
-        QStringLiteral("/KGlobalSettings"),
-        QStringLiteral("org.kde.KGlobalSettings"),
-        QStringLiteral("notifyChange"),
-        this,
-        SLOT(onGlobalSettingsChange(int,int))
-    );
+    connect(kcminputConfigWatcher.data(), &KConfigWatcher::configChanged, this, &GtkConfig::onKCMInputSettingsChange);
 
     ConfigEditor::removeLegacyGtk2Strings();
     applyAllSettings();
@@ -76,7 +69,7 @@ void GtkConfig::setGtk2Theme(const QString &themeName) const
 
 void GtkConfig::setGtk3Theme(const QString &themeName) const
 {
-    ConfigEditor::setGtk3ConfigValueDconf(QStringLiteral("gtk-theme"), themeName);
+    ConfigEditor::setGtk3ConfigValueGSettings(QStringLiteral("gtk-theme"), themeName);
     ConfigEditor::setGtk3ConfigValueSettingsIni(QStringLiteral("gtk-theme-name"), themeName);
     ConfigEditor::setGtk3ConfigValueXSettingsd(QStringLiteral("Net/ThemeName"),  themeName);
 }
@@ -105,34 +98,32 @@ void GtkConfig::setFont() const
 {
     const QString configFontName = configValueProvider->fontName();
     ConfigEditor::setGtk2ConfigValue(QStringLiteral("gtk-font-name"), configFontName);
-    ConfigEditor::setGtk3ConfigValueDconf(QStringLiteral("font-name"), configFontName);
+    ConfigEditor::setGtk3ConfigValueGSettings(QStringLiteral("font-name"), configFontName);
     ConfigEditor::setGtk3ConfigValueSettingsIni(QStringLiteral("gtk-font-name"), configFontName);
     ConfigEditor::setGtk3ConfigValueXSettingsd(QStringLiteral("Gtk/FontName"),  configFontName);
 }
 
-void GtkConfig::setIconTheme(int iconGroup) const
+void GtkConfig::setIconTheme() const
 {
-    if (iconGroup == KIconLoader::Group::Desktop) { // This is needed to update icons only once
-        const QString iconThemeName = configValueProvider->iconThemeName();
-        ConfigEditor::setGtk2ConfigValue(QStringLiteral("gtk-icon-theme-name"), iconThemeName);
-        ConfigEditor::setGtk3ConfigValueDconf(QStringLiteral("icon-theme"), iconThemeName);
-        ConfigEditor::setGtk3ConfigValueSettingsIni(QStringLiteral("gtk-icon-theme-name"), iconThemeName);
-        ConfigEditor::setGtk3ConfigValueXSettingsd(QStringLiteral("Net/IconThemeName"),  iconThemeName);
-    }
+    const QString iconThemeName = configValueProvider->iconThemeName();
+    ConfigEditor::setGtk2ConfigValue(QStringLiteral("gtk-icon-theme-name"), iconThemeName);
+    ConfigEditor::setGtk3ConfigValueGSettings(QStringLiteral("icon-theme"), iconThemeName);
+    ConfigEditor::setGtk3ConfigValueSettingsIni(QStringLiteral("gtk-icon-theme-name"), iconThemeName);
+    ConfigEditor::setGtk3ConfigValueXSettingsd(QStringLiteral("Net/IconThemeName"),  iconThemeName);
 }
 
 void GtkConfig::setCursorTheme() const
 {
     const QString cursorThemeName = configValueProvider->cursorThemeName();
     ConfigEditor::setGtk2ConfigValue(QStringLiteral("gtk-cursor-theme-name"), cursorThemeName);
-    ConfigEditor::setGtk3ConfigValueDconf(QStringLiteral("cursor-theme"), cursorThemeName);
+    ConfigEditor::setGtk3ConfigValueGSettings(QStringLiteral("cursor-theme"), cursorThemeName);
     ConfigEditor::setGtk3ConfigValueSettingsIni(QStringLiteral("gtk-cursor-theme-name"), cursorThemeName);
     ConfigEditor::setGtk3ConfigValueXSettingsd(QStringLiteral("Gtk/CursorThemeName"),  cursorThemeName);
 }
 
 void GtkConfig::setIconsOnButtons() const
 {
-    const QString iconsOnButtonsConfigValue = configValueProvider->iconsOnButtons();
+    const bool iconsOnButtonsConfigValue = configValueProvider->iconsOnButtons();
     ConfigEditor::setGtk2ConfigValue(QStringLiteral("gtk-button-images"), iconsOnButtonsConfigValue);
     ConfigEditor::setGtk3ConfigValueSettingsIni(QStringLiteral("gtk-button-images"), iconsOnButtonsConfigValue);
     ConfigEditor::setGtk3ConfigValueXSettingsd(QStringLiteral("Gtk/ButtonImages"), iconsOnButtonsConfigValue);
@@ -140,7 +131,7 @@ void GtkConfig::setIconsOnButtons() const
 
 void GtkConfig::setIconsInMenus() const
 {
-    const QString iconsInMenusConfigValue = configValueProvider->iconsInMenus();
+    const bool iconsInMenusConfigValue = configValueProvider->iconsInMenus();
     ConfigEditor::setGtk2ConfigValue(QStringLiteral("gtk-menu-images"), iconsInMenusConfigValue);
     ConfigEditor::setGtk3ConfigValueSettingsIni(QStringLiteral("gtk-menu-images"), iconsInMenusConfigValue);
     ConfigEditor::setGtk3ConfigValueXSettingsd(QStringLiteral("Gtk/MenuImages"), iconsInMenusConfigValue);
@@ -148,21 +139,16 @@ void GtkConfig::setIconsInMenus() const
 
 void GtkConfig::setToolbarStyle() const
 {
-    using ToolbarStyleNotation = ConfigValueProvider::ToolbarStyleNotation;
-
-    QString toolbarStyleSettingsIni = configValueProvider->toolbarStyle(ToolbarStyleNotation::SettingsIni);
-    QString toolbarStyleDConf = configValueProvider->toolbarStyle(ToolbarStyleNotation::Dconf);
-    QString toolbarStyleXSettingsd = configValueProvider->toolbarStyle(ToolbarStyleNotation::Xsettingsd);
-
-    ConfigEditor::setGtk2ConfigValue(QStringLiteral("gtk-toolbar-style"), toolbarStyleSettingsIni);
-    ConfigEditor::setGtk3ConfigValueDconf(QStringLiteral("toolbar-style"), toolbarStyleDConf);
-    ConfigEditor::setGtk3ConfigValueSettingsIni(QStringLiteral("gtk-toolbar-style"), toolbarStyleSettingsIni);
-    ConfigEditor::setGtk3ConfigValueXSettingsd(QStringLiteral("Gtk/ToolbarStyle"),  toolbarStyleXSettingsd);
+    const int toolbarStyle = configValueProvider->toolbarStyle();
+    ConfigEditor::setGtk2ConfigValue(QStringLiteral("gtk-toolbar-style"), toolbarStyle);
+    ConfigEditor::setGtk3ConfigValueGSettingsAsEnum(QStringLiteral("toolbar-style"), toolbarStyle);
+    ConfigEditor::setGtk3ConfigValueSettingsIni(QStringLiteral("gtk-toolbar-style"), toolbarStyle);
+    ConfigEditor::setGtk3ConfigValueXSettingsd(QStringLiteral("Gtk/ToolbarStyle"), toolbarStyle);
 }
 
 void GtkConfig::setScrollbarBehavior() const
 {
-    const QString scrollbarBehavior = configValueProvider->scrollbarBehavior();
+    const bool scrollbarBehavior = configValueProvider->scrollbarBehavior();
     ConfigEditor::setGtk2ConfigValue(QStringLiteral("gtk-primary-button-warps-slider"), scrollbarBehavior);
     ConfigEditor::setGtk3ConfigValueSettingsIni(QStringLiteral("gtk-primary-button-warps-slider"), scrollbarBehavior);
     ConfigEditor::setGtk3ConfigValueXSettingsd(QStringLiteral("Gtk/PrimaryButtonWarpsSlider"), scrollbarBehavior);
@@ -170,32 +156,37 @@ void GtkConfig::setScrollbarBehavior() const
 
 void GtkConfig::setDarkThemePreference() const
 {
-    const QString preferDarkTheme = configValueProvider->preferDarkTheme();
+    const bool preferDarkTheme = configValueProvider->preferDarkTheme();
     ConfigEditor::setGtk3ConfigValueSettingsIni(QStringLiteral("gtk-application-prefer-dark-theme"), preferDarkTheme);
 }
 
 void GtkConfig::setWindowDecorationsButtonsOrder() const
 {
     const QString windowDecorationsButtonOrder = configValueProvider->windowDecorationsButtonsOrder();
-    ConfigEditor::setGtk3ConfigValueDconf(QStringLiteral("button-layout"), windowDecorationsButtonOrder, QStringLiteral("org.gnome.desktop.wm.preferences"));
+    ConfigEditor::setGtk3ConfigValueGSettings(QStringLiteral("button-layout"), windowDecorationsButtonOrder, QStringLiteral("org.gnome.desktop.wm.preferences"));
     ConfigEditor::setGtk3ConfigValueSettingsIni(QStringLiteral("gtk-decoration-layout"), windowDecorationsButtonOrder);
     ConfigEditor::setGtk3ConfigValueXSettingsd(QStringLiteral("Gtk/DecorationLayout"), windowDecorationsButtonOrder);
 }
 
 void GtkConfig::setEnableAnimations() const
 {
-    const QString enableAnimations = configValueProvider->enableAnimations();
+    const bool enableAnimations = configValueProvider->enableAnimations();
     ConfigEditor::setGtk2ConfigValue(QStringLiteral("gtk-enable-animations"), enableAnimations);
-    // FIXME work with booleans in configValueProvider...
-    ConfigEditor::setGtk3ConfigValueDconf(QStringLiteral("enable-animations"), enableAnimations == QLatin1String("1"));
+    ConfigEditor::setGtk3ConfigValueGSettings(QStringLiteral("enable-animations"), enableAnimations);
     ConfigEditor::setGtk3ConfigValueSettingsIni(QStringLiteral("gtk-enable-animations"), enableAnimations);
     ConfigEditor::setGtk3ConfigValueXSettingsd(QStringLiteral("Gtk/EnableAnimations"), enableAnimations);
 }
 
+void GtkConfig::setColors() const
+{
+    const QMap<QString, QColor> colors = configValueProvider->colors();
+    ConfigEditor::setGtk3Colors(colors);
+}
+
 void GtkConfig::applyAllSettings() const
 {
     setFont();
-    setIconTheme(KIconLoader::Group::Desktop);
+    setIconTheme();
     setCursorTheme();
     setIconsOnButtons();
     setIconsInMenus();
@@ -204,31 +195,40 @@ void GtkConfig::applyAllSettings() const
     setDarkThemePreference();
     setWindowDecorationsButtonsOrder();
     setEnableAnimations();
-}
-
-void GtkConfig::onGlobalSettingsChange(int settingsChangeType, int arg) const
-{
-    SettingsChangeType changeType = static_cast<SettingsChangeType>(settingsChangeType);
-    SettingsCategory settingsCategory = static_cast<SettingsCategory>(arg);
-
-    if (changeType == SettingsChangeType::Cursor) {
-        setCursorTheme();
-    } else if (changeType == SettingsChangeType::Settings && settingsCategory == SettingsCategory::Style) {
-        setIconsOnButtons();
-        setIconsInMenus();
-        setToolbarStyle();
-    } else if (changeType == SettingsChangeType::Settings && settingsCategory == SettingsCategory::Mouse) {
-        setScrollbarBehavior();
-    } else if (changeType == SettingsChangeType::Palette) {
-        setDarkThemePreference();
-    }
+    setColors();
 }
 
 void GtkConfig::onKdeglobalsSettingsChange(const KConfigGroup &group, const QByteArrayList &names) const
 {
-    if (group.name() == QLatin1String("KDE")
-            && names.contains(QByteArrayLiteral("AnimationDurationFactor"))) {
-        setEnableAnimations();
+    if (group.name() == QStringLiteral("KDE")) {
+        if (names.contains(QByteArrayLiteral("AnimationDurationFactor"))) {
+            setEnableAnimations();
+        }
+        if (names.contains(QByteArrayLiteral("ShowIconsInMenuItems"))) {
+            setIconsInMenus();
+        }
+        if (names.contains(QByteArrayLiteral("ShowIconsOnPushButtons"))) {
+            setIconsOnButtons();
+        }
+        if (names.contains(QByteArrayLiteral("ScrollbarLeftClickNavigatesByPage"))) {
+            setScrollbarBehavior();
+        }
+    } else if (group.name() == QStringLiteral("Icons")) {
+        if (names.contains(QByteArrayLiteral("Theme"))) {
+            setIconTheme();
+        }
+    } else if (group.name() == QStringLiteral("General")) {
+        if (names.contains(QByteArrayLiteral("font"))) {
+            setFont();
+        }
+        if (names.contains(QByteArrayLiteral("ColorScheme"))) {
+            setColors();
+            setDarkThemePreference();
+        }
+    } else if (group.name() == QStringLiteral("Toolbar style")) {
+        if (names.contains(QByteArrayLiteral("ToolButtonStyle"))) {
+            setToolbarStyle();
+        }
     }
 }
 
@@ -240,4 +240,13 @@ void GtkConfig::onKWinSettingsChange(const KConfigGroup &group, const QByteArray
     }
 }
 
+void GtkConfig::onKCMInputSettingsChange(const KConfigGroup& group, const QByteArrayList& names) const
+{
+    if (group.name() == QStringLiteral("Mouse")
+            && names.contains("cursorTheme")) {
+        setCursorTheme();
+    }
+}
+
+
 #include "gtkconfig.moc"
